# -*- coding: utf-8 -*-
import sys, re, json
from bs4 import BeautifulSoup
try:
    from urllib2 import urlopen
except ImportError:
    from urllib.request import urlopen
try:
    from urlparse import urlparse, parse_qs, urljoin
except ImportError:
    from urllib.parse import urlparse, parse_qs, urljoin


def getsoup(content):
    if (2, 7, 3) <= sys.version_info < (3, 0) or (3, 2) <= sys.version_info < (4, 0):
        return BeautifulSoup(content, 'html.parser')
    return BeautifulSoup(content, 'html5lib')


def getjson(url):
    return json.load(urlopen(url))


def gethttp(url):
    return urlopen(url, timeout=3).read().decode('utf-8')


def playlist(url):
    try:
        soup = getsoup(gethttp(url))
    except Exception as e:
        return {'code': 'youtube 200', 'error': str(e)}
    ret = []
    for e in soup.find_all('td', {'class': 'pl-video-title'}):
        ret.append({'title': e.a.text.strip(), 'url': urljoin(url, e.a['href'])})
    e = soup.find('button', {'class': 'browse-items-load-more-button'})
    if e:
        for k, v in e.attrs.items():
            if k[-4:] == 'href':
                ret.append({'title': 'Load more', 'url': urljoin(url, v)})
    return ret


def uservideos(url):
    try:
        soup = getsoup(gethttp(url))
    except Exception as e:
        return {'code': 'youtube 400', 'error': str(e)}
    ret = []
    for e in soup.find_all('h3', {'class': 'yt-lockup-title'}):
        ret.append({'title': e.a.text.strip(), 'url': urljoin(url, e.a['href'])})
    e = soup.find('button', {'class': 'browse-items-load-more-button'})
    if e:
        for k, v in e.attrs.items():
            if k[-4:] == 'href':
                ret.append({'title': 'Load more', 'url': urljoin(url, v)})
    return ret


def user(url):
    return [{'title': 'videos', 'url': url + '/videos'}, {'title': 'playlists', 'url': url + '/playlists?view=1'}]


def loadmore(url):
    try:
        data = getjson(url)
    except Exception as e:
        return {'code': 'youtube 300', 'error': str(e)}
    try:
        soup = getsoup(data['content_html'])
    except Exception as e:
        return {'code': 'youtube 301', 'error': str(e)}
    ret = []
    for e in soup.find_all('h3', {'class': 'yt-lockup-title'}):
        ret.append({'title': e.a.text.strip(), 'url': urljoin(url, e.a['href'])})
    for e in soup.find_all('td', {'class': 'pl-video-title'}):
        ret.append({'title': e.a.text.strip(), 'url': urljoin(url, e.a['href'])})
    try:
        soup = getsoup(data['load_more_widget_html'])
    except Exception as e:
        return {'code': 'youtube 302', 'error': str(e)}
    e = soup.find('button', {'class': 'browse-items-load-more-button'})
    if e:
        for k, v in e.attrs.items():
            if k[-4:] == 'href':
                ret.append({'title': 'Load more', 'url': urljoin(url, v)})
    return ret


def match1(text, *patterns):
    if len(patterns) == 1:
        pattern = patterns[0]
        match = re.search(pattern, text)
        if match:
            return match.group(1)
        else:
            return None
    else:
        ret = []
        for pattern in patterns:
            match = re.search(pattern, text)
            if match:
                ret.append(match.group(1))
        return ret


def decipher(js, s):
    def tr_js(code):
        code = re.sub(r'function', r'def', code)
        code = re.sub(r'(\W)(as|if|in|is|or)\(', r'\1_\2(', code)
        code = re.sub(r'\$', '_dollar', code)
        code = re.sub(r'\{', r':\n\t', code)
        code = re.sub(r'\}', r'\n', code)
        code = re.sub(r'var\s+', r'', code)
        code = re.sub(r'(\w+).join\(""\)', r'"".join(\1)', code)
        code = re.sub(r'(\w+).length', r'len(\1)', code)
        code = re.sub(r'(\w+).slice\((\w+)\)', r'\1[\2:]', code)
        code = re.sub(r'(\w+).splice\((\w+),(\w+)\)', r'del \1[\2:\2+\3]', code)
        code = re.sub(r'(\w+).split\(""\)', r'list(\1)', code)
        return code

    js = js.replace('\n', ' ')
    f1 = match1(js, r'\.set\(\w+\.sp,encodeURIComponent\(([$\w]+)') or \
         match1(js, r'\.set\(\w+\.sp,\(0,window\.encodeURIComponent\)\(([$\w]+)') or \
         match1(js, r'\.set\(\w+\.sp,([$\w]+)\(\w+\.s\)\)') or \
         match1(js, r'"signature",([$\w]+)\(\w+\.\w+\)')
    f1def = match1(js, r'function %s(\(\w+\)\{[^\{]+\})' % re.escape(f1)) or \
            match1(js, r'\W%s=function(\(\w+\)\{[^\{]+\})' % re.escape(f1))
    f1def = re.sub(r'([$\w]+\.)([$\w]+\(\w+,\d+\))', r'\2', f1def)
    f1def = 'function main_%s%s' % (f1, f1def)  # prefix to avoid potential namespace conflict
    code = tr_js(f1def)
    f2s = set(re.findall(r'([$\w]+)\(\w+,\d+\)', f1def))
    for f2 in f2s:
        f2e = re.escape(f2)
        f2def = re.search(r'[^$\w]%s:function\((\w+,\w+)\)(\{[^\{\}]+\})' % f2e, js)
        if f2def:
            f2def = 'function {}({}){}'.format(f2e, f2def.group(1), f2def.group(2))
        else:
            f2def = re.search(r'[^$\w]%s:function\((\w+)\)(\{[^\{\}]+\})' % f2e, js)
            f2def = 'function {}({},b){}'.format(f2e, f2def.group(1), f2def.group(2))
        f2 = re.sub(r'(as|if|in|is|or)', r'_\1', f2)
        f2 = re.sub(r'\$', '_dollar', f2)
        code = code + 'global %s\n' % f2 + tr_js(f2def)

    f1 = re.sub(r'(as|if|in|is|or)', r'_\1', f1)
    f1 = re.sub(r'\$', '_dollar', f1)
    code = code + 'sig=main_%s(s)' % f1  # prefix to avoid potential namespace conflict
    exec (code, globals(), locals())
    return locals()['sig']


def play3(url):
    content = gethttp(url)
    if not content:
        return {'code': 'youtube 100'}
    m = re.search(r'ytplayer.config\s*=\s*({[^\n]+?});', content)
    if not m:
        return {'code': 'youtube 101'}
    content = m.group(1)
    try:
        data1 = json.loads(content)
    except Exception as e:
        return {'code': 'youtube 102', 'error': str(e)}
    try:
        data2 = json.loads(data1['args']['player_response'])
    except Exception as e:
        return {'code': 'youtube 103', 'error': str(e)}
    urls = {}
    for elem in data2['streamingData']['formats']:
        if 'cipher' in elem:
            url = 'https://www.youtube.com' + data1['assets']['js']
            contents = gethttp(url)
            if not contents:
                return {'code': 'youtube 104'}
            para = parse_qs(elem['cipher'])
            sig = decipher(contents, para['s'][0])
            urls[elem['mimeType']] = para['url'][0] + '&signature=' + sig
        else:
            urls[elem['mimeType']] = elem['url']
    return {'title': data2['videoDetails']['title'], 'url': urls}
